@@ -28,6 +28,7 @@ __license__   = "LGPL"
 import pyatspi
 
 from . import cmdnames
+from . import debug
 from . import guilabels
 from . import input_event
 from . import keybindings
@@ -35,7 +36,6 @@ from . import messages
 from . import orca_state
 from . import settings
 from . import settings_manager
-from . import speech
 
 _settingsManager = settings_manager.getManager()
 
@@ -583,7 +583,7 @@ class Chat:
         # Only speak/braille the new message if it matches how the user
         # wants chat messages spoken.
         #
-        verbosity = self._script.getSettings().chatMessageVerbosity
+        verbosity = _settingsManager.getAppSetting(self._script.app, 'chatMessageVerbosity')
         if orca_state.activeScript.name != self._script.name \
            and verbosity == settings.CHAT_SPEAK_ALL_IF_FOCUSED:
             return
@@ -596,7 +596,8 @@ class Chat:
         text = self._script.utilities.appendString(text, message)
 
         if len(text.strip()):
-            speech.speak(text)
+            voice = self._script.speechGenerator.voice(string=text)
+            self._script.speakMessage(text, voice=voice)
         self._script.displayBrailleMessage(text)
 
     def getMessageFromEvent(self, event):
@@ -677,11 +678,8 @@ class Chat:
 
         elif self.isAutoCompletedTextEvent(event):
             text = event.any_data
-            if text.isupper():
-                speech.speak(text,
-                             self._script.voices[settings.UPPERCASE_VOICE])
-            else:
-                speech.speak(text)
+            voice = self._script.speechGenerator.voice(string=text)
+            self._script.speakMessage(text, voice=voice)
             return True
 
         return False
@@ -700,7 +698,8 @@ class Chat:
         if _settingsManager.getSetting('chatAnnounceBuddyTyping'):
             conversation = self.getConversation(event.source)
             if conversation and (status != conversation.getTypingStatus()):
-                speech.speak(status)
+                voice = self._script.speechGenerator.voice(string=status)
+                self._script.speakMessage(status, voice=voice)
                 conversation.setTypingStatus(status)
                 return True
 
@@ -855,10 +854,13 @@ class Chat:
         """
 
         if obj and obj.getState().contains(pyatspi.STATE_SHOWING):
-            topLevel = self._script.utilities.topLevelObject(obj)
-            if topLevel and topLevel.getState().contains(pyatspi.STATE_ACTIVE):
-                return True
+            active = self._script.utilities.topLevelObjectIsActiveAndCurrent(obj)
+            msg = "INFO: %s's window is focused chat: %s" % (obj, active)
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return active
 
+        msg = "INFO: %s is not focused chat (not showing)" % obj
+        debug.println(debug.LEVEL_INFO, msg, True)
         return False
 
     def getChatRoomName(self, obj):
