@@ -37,6 +37,7 @@ class ScriptManager:
         debug.println(debug.LEVEL_INFO, 'SCRIPT MANAGER: Initializing', True)
         self.appScripts = {}
         self.toolkitScripts = {}
+        self.customScripts = {}
         self._appModules = apps.__all__
         self._toolkitModules = toolkits.__all__
         self._defaultScript = None
@@ -62,8 +63,6 @@ class ScriptManager:
              'gnome-calculator': 'gcalctool',
              'marco':            'metacity',
              'Nereid':           'Banshee',
-             'vte':              'gnome-terminal',
-             'gnome-terminal-server': 'gnome-terminal',
              'mate-notification-daemon': 'notification-daemon',
             }
 
@@ -90,6 +89,7 @@ class ScriptManager:
         self.setActiveScript(None, "deactivate")
         self.appScripts = {}
         self.toolkitScripts = {}
+        self.customScripts = {}
         debug.println(debug.LEVEL_INFO, 'SCRIPT MANAGER: Deactivated', True)
 
     def getModuleName(self, app):
@@ -139,6 +139,17 @@ class ScriptManager:
 
         return name
 
+    def _scriptForRole(self, obj):
+        try:
+            role = obj.getRole()
+        except:
+            return ''
+
+        if role == pyatspi.ROLE_TERMINAL:
+            return 'terminal'
+
+        return ''
+
     def _newNamedScript(self, app, name):
         """Attempts to locate and load the named module. If successful, returns
         a script based on this module."""
@@ -221,9 +232,19 @@ class ScriptManager:
         Returns an instance of a Script.
         """
 
+        customScript = None
         appScript = None
         toolkitScript = None
 
+        roleName = self._scriptForRole(obj)
+        if roleName:
+            customScripts = self.customScripts.get(app, {})
+            customScript = customScripts.get(roleName)
+            if not customScript:
+                customScript = self._newNamedScript(app, roleName)
+                customScripts[roleName] = customScript
+            self.customScripts[app] = customScripts
+
         objToolkit = self._toolkitForObject(obj)
         if objToolkit:
             toolkitScripts = self.toolkitScripts.get(app, {})
@@ -247,6 +268,9 @@ class ScriptManager:
             debug.println(debug.LEVEL_WARNING, msg, True)
             appScript = self.getDefaultScript()
 
+        if customScript:
+            return customScript
+
         # Only defer to the toolkit script for this object if the app script
         # is based on a different toolkit.
         if toolkitScript \
@@ -301,6 +325,14 @@ class ScriptManager:
                 for toolkitScript in toolkitScripts.values():
                     del toolkitScript
 
+            try:
+                customScripts = self.customScripts.pop(app)
+            except KeyError:
+                pass
+            else:
+                for customScript in customScripts.values():
+                    del customScript
+
             del app
 
 _manager = ScriptManager()
