@@ -79,6 +79,7 @@ class Script(default.Script):
         self._inMouseOverObject = False
         self._inFocusMode = False
         self._focusModeIsSticky = False
+        self._browseModeIsSticky = False
 
         if _settingsManager.getSetting('caretNavigationEnabled') is None:
             _settingsManager.setSetting('caretNavigationEnabled', True)
@@ -152,6 +153,14 @@ class Script(default.Script):
                 self.inputEventHandlers.get("enableStickyFocusModeHandler"),
                 2))
 
+        keyBindings.add(
+            keybindings.KeyBinding(
+                "a",
+                keybindings.defaultModifierMask,
+                keybindings.ORCA_MODIFIER_MASK,
+                self.inputEventHandlers.get("enableStickyBrowseModeHandler"),
+                3))
+
         layout = _settingsManager.getSetting('keyboardLayout')
         if layout == settings.GENERAL_KEYBOARD_LAYOUT_DESKTOP:
             key = "KP_Multiply"
@@ -212,6 +221,10 @@ class Script(default.Script):
                 Script.enableStickyFocusMode,
                 cmdnames.SET_FOCUS_MODE_STICKY)
 
+        self.inputEventHandlers["enableStickyBrowseModeHandler"] = \
+            input_event.InputEventHandler(
+                Script.enableStickyBrowseMode,
+                cmdnames.SET_BROWSE_MODE_STICKY)
 
     def getBookmarks(self):
         """Returns the "bookmarks" class for this script."""
@@ -682,12 +695,20 @@ class Script(default.Script):
 
         return self._focusModeIsSticky
 
+    def browseModeIsSticky(self):
+        """Returns True if we're in 'sticky' browse mode."""
+
+        return self._browseModeIsSticky
+
     def useFocusMode(self, obj):
         """Returns True if we should use focus mode in obj."""
 
         if self._focusModeIsSticky:
             return True
 
+        if self._browseModeIsSticky:
+            return False
+
         if not _settingsManager.getSetting('structNavTriggersFocusMode') \
            and self._lastCommandWasStructNav:
             return False
@@ -752,7 +773,8 @@ class Script(default.Script):
         obj, offset = self.utilities.getCaretContext(documentFrame=None)
         self.speakContents(self.utilities.getLineContentsAtOffset(obj, offset))
 
-    def presentObject(self, obj, offset=0):
+    def presentObject(self, obj, **args):
+        offset = args.get("offset", 0)
         contents = self.utilities.getObjectContentsAtOffset(obj, offset)
         self.displayContents(contents)
         self.speakContents(contents)
@@ -954,12 +976,21 @@ class Script(default.Script):
         self._inMouseOverObject = False
         self._lastMouseOverObject = None
 
+    def enableStickyBrowseMode(self, inputEvent, forceMessage=False):
+        if not self._browseModeIsSticky or forceMessage:
+            self.presentMessage(messages.MODE_BROWSE_IS_STICKY)
+
+        self._inFocusMode = False
+        self._focusModeIsSticky = False
+        self._browseModeIsSticky = True
+
     def enableStickyFocusMode(self, inputEvent, forceMessage=False):
         if not self._focusModeIsSticky or forceMessage:
             self.presentMessage(messages.MODE_FOCUS_IS_STICKY)
 
         self._inFocusMode = True
         self._focusModeIsSticky = True
+        self._browseModeIsSticky = False
 
     def togglePresentationMode(self, inputEvent):
         [obj, characterOffset] = self.utilities.getCaretContext()
@@ -984,6 +1015,7 @@ class Script(default.Script):
             self.presentMessage(messages.MODE_FOCUS)
         self._inFocusMode = not self._inFocusMode
         self._focusModeIsSticky = False
+        self._browseModeIsSticky = False
 
     def locusOfFocusChanged(self, event, oldFocus, newFocus):
         """Handles changes of focus of interest to the script."""
@@ -1052,6 +1084,7 @@ class Script(default.Script):
             return True
 
         if not self._focusModeIsSticky \
+           and not self._browseModeIsSticky \
            and self.useFocusMode(newFocus) != self._inFocusMode:
             self.togglePresentationMode(None)
 
@@ -1105,10 +1138,11 @@ class Script(default.Script):
         if not obj or self.utilities.isZombie(obj):
             self.utilities.clearCaretContext()
 
+        shouldPresent = self.utilities.isShowingOrVisible(event.source)
         if not _settingsManager.getSetting('onlySpeakDisplayedText'):
             if event.detail1:
                 self.presentMessage(messages.PAGE_LOADING_START)
-            elif event.source.name:
+            elif event.source.name and shouldPresent:
                 msg = messages.PAGE_LOADING_END_NAMED % event.source.name
                 self.presentMessage(msg, resetStyles=False)
             else:
@@ -1119,7 +1153,7 @@ class Script(default.Script):
 
         self.utilities.clearCachedObjects()
 
-        if _settingsManager.getSetting('pageSummaryOnLoad'):
+        if _settingsManager.getSetting('pageSummaryOnLoad') and shouldPresent:
             msg = "WEB: Getting page summary for obj %s" % obj
             debug.println(debug.LEVEL_INFO, msg, True)
             summary = self.utilities.getPageSummary(obj)
@@ -1434,8 +1468,14 @@ class Script(default.Script):
                 self.utilities.setCaretContext(replicant, offset)
                 return True
 
-        child = event.any_data
-        if child.getRole() in [pyatspi.ROLE_ALERT, pyatspi.ROLE_DIALOG]:
+        childRole = event.any_data.getRole()
+        if childRole == pyatspi.ROLE_ALERT:
+            msg = "WEB: Presenting event.any_data"
+            debug.println(debug.LEVEL_INFO, msg, True)
+            self.presentObject(event.any_data)
+            return True
+
+        if childRole == pyatspi.ROLE_DIALOG:
             msg = "WEB: Setting locusOfFocus to event.any_data"
             debug.println(debug.LEVEL_INFO, msg, True)
             orca.setLocusOfFocus(event, child)
@@ -1446,7 +1486,7 @@ class Script(default.Script):
             utterances.append(messages.NEW_ITEM_ADDED)
             utterances.extend(self.speechGenerator.generateSpeech(child, force=True))
             speech.speak(utterances)
-            self._lastMouseOverObject = child
+            self._lastMouseOverObject = event.any_data
             self.preMouseOverContext = self.utilities.getCaretContext()
             return True
 
@@ -1717,6 +1757,12 @@ class Script(default.Script):
         debug.println(debug.LEVEL_INFO, msg, True)
         self.utilities.clearContentCache()
 
+        document = self.utilities.getDocumentForObject(event.source)
+        if document:
+            msg = "WEB: Clearing structural navigation cache for %s" % document
+            debug.println(debug.LEVEL_INFO, msg, True)
+            self.structuralNavigation.clearCache(document)
+
         state = event.source.getState()
         if not state.contains(pyatspi.STATE_EDITABLE):
             if self._inMouseOverObject \
@@ -1776,6 +1822,12 @@ class Script(default.Script):
             self.liveRegionManager.handleEvent(event)
             return True
 
+        document = self.utilities.getDocumentForObject(event.source)
+        if document:
+            msg = "WEB: Clearing structural navigation cache for %s" % document
+            debug.println(debug.LEVEL_INFO, msg, True)
+            self.structuralNavigation.clearCache(document)
+
         text = self.utilities.queryNonEmptyText(event.source)
         if not text:
             msg = "WEB: Ignoring: Event source is not a text object"
