@@ -93,6 +93,7 @@ userCustomizableSettings = [
     "beepProgressBarUpdates",
     "progressBarUpdateInterval",
     "progressBarVerbosity",
+    "ignoreStatusBarProgressBars",
     "enableContractedBraille",
     "brailleContractionTable",
     "enableMouseReview",
@@ -281,15 +282,17 @@ presentLockingKeys           = None
 
 # Mouse review
 enableMouseReview          = False
-mouseDwellDelay            = 0
-mouseDwellMaxDrift         = 3
 
 # Progressbars
 speakProgressBarUpdates    = True
 brailleProgressBarUpdates  = False
 beepProgressBarUpdates     = False
 progressBarUpdateInterval  = 10
+progressBarSpeechInterval  = None
+progressBarBrailleInterval = None
+progressBarBeepInterval    = 0
 progressBarVerbosity       = PROGRESS_BAR_APPLICATION
+ignoreStatusBarProgressBars = True
 
 # Structural navigation
 structuralNavigationEnabled = True
